/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Restrictions;

import com.patrikdufresne.managers.AbstractManager;
import com.patrikdufresne.managers.ArchivableObject;
import com.patrikdufresne.managers.ManagerContext;
import com.patrikdufresne.managers.ManagerException;
import com.patrikdufresne.managers.Query;

/**
 */
public class NonAvailabilityManager extends AbstractManager<NonAvailability> {
    /**
     * Create a time range criterion.
     * 
     * @param start
     *            the start time
     * @param end
     *            the end time
     * @return the Criterion.
     */
    private static Criterion startEndCriterion(Date start, Date end) {
        // (start < event.start < end)
        // OU
        // (start < event.end < end)
        // OU
        // (event.start < start ET event.end > end)
        return Restrictions.or(Restrictions.or(Restrictions.between(AbstractCalendarEvent.START_DATE, start, end), Restrictions.between(
                AbstractCalendarEvent.END_DATE,
                start,
                end)), Restrictions.and(Restrictions.lt(AbstractCalendarEvent.START_DATE, start), Restrictions.gt(AbstractCalendarEvent.END_DATE, end)));
    }

    public NonAvailabilityManager(PlanimodManagers managers) {
        super(managers);
    }

    /**
     * Return the managers
     * 
     * @return
     */
    @Override
    public PlanimodManagers getManagers() {
        return (PlanimodManagers) super.getManagers();
    }

    /**
     * List all non-availabilities for all employee for the given time range.
     * <p>
     * This list does not contrains archived employee.
     * 
     * @throws ManagerException
     */
    public List<NonAvailability> list(final Date start, final Date end) throws ManagerException {
        return getManagers().query(new Query<List<NonAvailability>>() {
            @Override
            public List<NonAvailability> run() throws ManagerException {
                return ManagerContext.getDefaultSession().createCriteria(NonAvailability.class).add(startEndCriterion(start, end)).createCriteria(
                        NonAvailability.EMPLOYEE).add(Restrictions.isNull(ArchivableObject.ARCHIVED_DATE)).list();
            }
        });
    }

    /**
     * List the non-availabilities for the given time range and visibility.
     * 
     * @param start
     *            the start date
     * @param end
     *            the end date
     * @param visibility
     *            True to show visible item.
     * @return list of non-availabilities.
     * @throws ManagerException
     */
    public List<NonAvailability> listByVisibility(final Date start, final Date end, final boolean visibility) throws ManagerException {
        return getManagers().query(new Query<List<NonAvailability>>() {
            @Override
            public List<NonAvailability> run() throws ManagerException {
                return ManagerContext.getDefaultSession().createCriteria(NonAvailability.class).add(Restrictions.eq(NonAvailability.VISIBLE, visibility)).add(
                        startEndCriterion(start, end)).createCriteria(NonAvailability.EMPLOYEE).add(Restrictions.isNull(ArchivableObject.ARCHIVED_DATE)).list();
            }
        });
    }

    public List<NonAvailability> listByEmployees(final Collection<Employee> employees, final Date start, final Date end) throws ManagerException {
        return getManagers().query(new Query<List<NonAvailability>>() {
            @Override
            public List<NonAvailability> run() throws ManagerException {
                return ManagerContext.getDefaultSession().createCriteria(objectClass()).add(startEndCriterion(start, end)).add(
                        Restrictions.in(NonAvailability.EMPLOYEE, employees)).list();
            }
        });
    }

    @Override
    public Class<NonAvailability> objectClass() {
        return NonAvailability.class;
    }

    /**
     * This implementation check the starting and ending date value.
     */
    @Override
    protected void preAddObject(NonAvailability t) throws ManagerException {
        // Check date value
        if (t.getStartDate() != null && t.getEndDate() != null && t.getStartDate().compareTo(t.getEndDate()) > 0) {
            throw new IntegrityException("start > end"); //$NON-NLS-1$
        }
        // Check date precision
        Calendar cal = Calendar.getInstance();
        cal.setTime(t.getStartDate());
        if (cal.get(Calendar.MILLISECOND) != 0 || cal.get(Calendar.SECOND) != 0) {
            throw new IntegrityException("startDate too precise"); //$NON-NLS-1$
        }
        cal.setTime(t.getEndDate());
        if (cal.get(Calendar.MILLISECOND) != 0 || cal.get(Calendar.SECOND) != 0) {
            throw new IntegrityException("endDate too precise"); //$NON-NLS-1$
        }
    }

    /**
     * This implementation check the starting and ending date value.
     */
    @Override
    protected void preUpdateObject(NonAvailability t) throws ManagerException {
        // Check date value
        if (t.getStartDate() != null && t.getEndDate() != null && t.getStartDate().compareTo(t.getEndDate()) > 0) {
            throw new IntegrityException("start > end"); //$NON-NLS-1$
        }
        // Check date precision
        Calendar cal = Calendar.getInstance();
        cal.setTime(t.getStartDate());
        if (cal.get(Calendar.MILLISECOND) != 0 || cal.get(Calendar.SECOND) != 0) {
            throw new IntegrityException("startDate too precise"); //$NON-NLS-1$
        }
        cal.setTime(t.getEndDate());
        if (cal.get(Calendar.MILLISECOND) != 0 || cal.get(Calendar.SECOND) != 0) {
            throw new IntegrityException("endDate too precise"); //$NON-NLS-1$
        }
    }
}
