/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;

import com.patrikdufresne.managers.AbstractManager;
import com.patrikdufresne.managers.Exec;
import com.patrikdufresne.managers.ManagerContext;
import com.patrikdufresne.managers.ManagerException;
import com.patrikdufresne.managers.Query;
import com.patrikdufresne.util.BidiMultiHashMap;
import com.patrikdufresne.util.BidiMultiMap;

//TODO is a qualification is removed, make sure to un-assigned the preferred position.

/**
 * Object manager for qualification.
 * 
 * @author Patrik Dufresne
 * 
 */
public class QualificationManager extends AbstractManager<Qualification> {

    public QualificationManager(PlanimodManagers managers) {
        super(managers);
    }

    /**
     * This function is used to set the qualification for a given pair of
     * employee-position.
     * 
     * @param position
     *            the position
     * @param employee
     *            the employee
     * @param qualify
     *            True if the employee is qualify, false otherwise
     * @throws ManagerException
     */
    public void setQualification(final Position position, final Employee employee, final boolean qualify) throws ManagerException {
        getManagers().exec(new Exec() {
            @Override
            public void run() throws ManagerException {

                Qualification q = listByPositionAndEmployee(position, employee);
                if (q == null && qualify) {
                    Qualification qualif = new Qualification();
                    qualif.setEmployee(employee);
                    qualif.setPosition(position);
                    add(Arrays.asList(qualif));
                } else if (q != null && !qualify) {
                    remove(Arrays.asList(q));
                }

            }
        });
    }

    /**
     * Check if the qualification already exists.
     */
    @Override
    protected void preAddObject(Qualification q) throws ManagerException {
        if (listByPositionAndEmployee(q.getPosition(), q.getEmployee()) != null) {
            throw new IntegrityException("Qualification already exists."); //$NON-NLS-1$
        }
    }

    /**
     * List qualification objects matching the given position and employee.
     * 
     * @param position
     *            the position
     * @param employee
     *            the employee
     * @throws ManagerException
     */
    protected Qualification listByPositionAndEmployee(final Position position, final Employee employee) throws ManagerException {
        return getManagers().query(new Query<Qualification>() {
            @Override
            public Qualification run() throws ManagerException {
                return (Qualification) ManagerContext
                        .getDefaultSession()
                        .createCriteria(objectClass())
                        .add(Restrictions.eq(Qualification.POSITION, position))
                        .add(Restrictions.eq(Qualification.EMPLOYEE, employee))
                        .uniqueResult();
            }
        });
    }

    /**
     * List of position for which the given employees are qualify for.
     * 
     * @param employee
     *            the employees to check
     * @return list of position
     * @throws ManagerException
     */
    public List<Position> listQualifyPositionForEmployee(final Collection<Employee> employees) throws ManagerException {
        return getManagers().query(new Query<List<Position>>() {
            @Override
            public List<Position> run() throws ManagerException {
                return ManagerContext
                        .getDefaultSession()
                        .createCriteria(Qualification.class)
                        .add(Restrictions.in(Qualification.EMPLOYEE, employees))
                        .setProjection(Projections.distinct(Projections.property(Qualification.POSITION)))
                        .list();
            }
        });
    }

    /**
     * Create a BidiMultiMap to represent the qualification of the requested
     * employees and positions.
     * <p>
     * The table contains only not archived employee.
     * 
     * @param employees
     *            the employees
     * @param position
     *            the positions
     * @return the map
     * @throws ManagerException
     */
    public BidiMultiMap<Employee, Position> getQualificationTableByPositions(final Collection<Position> positions) throws ManagerException {

        if (positions.size() == 0) {
            return BidiMultiMap.EMPTY;
        }

        return getManagers().query(new Query<BidiMultiMap<Employee, Position>>() {
            @SuppressWarnings("unchecked")
            @Override
            public BidiMultiMap<Employee, Position> run() throws ManagerException {

                org.hibernate.Query namedQuery = ManagerContext.getDefaultSession().getNamedQuery("getQualificationTableByPositions"); //$NON-NLS-1$
                namedQuery.setParameterList("positions", positions); //$NON-NLS-1$

                BidiMultiMap<Employee, Position> table = new BidiMultiHashMap<Employee, Position>();
                for (Object[] obj : (Collection<Object[]>) namedQuery.list()) {
                    table.put((Employee) obj[0], (Position) obj[1]);
                }
                return table;

            }
        });

    }

    @Override
    public Class<Qualification> objectClass() {
        return Qualification.class;
    }

    /**
     * This function is used to remove any qualification catching the given pair
     * of employee-position.
     * 
     * @param position
     *            the position
     * @param employee
     *            the employee
     * @throws ManagerException
     */
    public void removeQualification(final Position position, final Employee employee) throws ManagerException {
        getManagers().exec(new Exec() {
            @Override
            public void run() throws ManagerException {
                Qualification qualif = listByPositionAndEmployee(position, employee);
                remove(Arrays.asList(qualif));
            }
        });
    }

}
