/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.patrikdufresne.ilp.Linear;
import com.patrikdufresne.ilp.util.Constraints;

/**
 * This runnable check if the problem is feasible without any swappable tasks.
 * 
 * <pre>
 * SUM from{i=0} to{n} SUM from{j=0} to{m} SUM from{k notin RTj} xijk ==0
 * </pre>
 * 
 * @author Patrik Dufresne
 */
public class CheckSwappableTask extends AbstractConstraintRunnable {

    private static final transient Logger LOGGER = LoggerFactory.getLogger(CheckSwappableTask.class);

    /**
     * The constraint's name
     */
    private static final String CONSTRAINT_NAME = "SUM from{i=0} to{n} SUM from{j=0} to{m} SUM from{k notin RTj} xijk <=0"; //$NON-NLS-1$

    /**
     * Public constructor.
     * 
     * @param problem
     *            reference to ProblemBuilder
     */
    public CheckSwappableTask(ProblemBuilder problem) {
        super(problem);
        setFeasibilityPumpHeuristic(true);
    }

    @Override
    protected boolean setUp() {
        // Create a linear function to compute a value using the snapshot.
        Linear linear = pb.lp.createLinear();
        int m = pb.table.tj().size();
        int p = pb.tasks.size();
        for (int k = 0; k < p; k++) {
            for (int j = 0; j < m; j++) {
                if (!pb.table.kInRTj(k, j)) {
                    for (int i = 0; i < pb.xijk.length; i++) {
                        if (pb.xijk[i][j][k] != null) {
                            linear.add(pb.lp.createTerm(ONE, pb.xijk[i][j][k]));
                        }
                    }
                }
            }
        }
        // Create the constraint
        this.constraint = pb.lp.addConstraint(CONSTRAINT_NAME, linear, null, ZERO);
        // If the linear function is empty, their is not need to optimize
        // the problem.
        // If the last snapshot was feasible without swappable tasks, we
        // don't need to optimize either.
        if (pb.snapshot != null && Constraints.isSatisfied(this.constraint, pb.snapshot)) {
            this.feasible = true;
            return false;
        }
        // Fix all swappable tasks variableto zero
        pb.lp.setObjectiveLinear(null);
        // Check feasibility without swappable tasks
        LOGGER.info("Check feasibility without swappable tasks."); //$NON-NLS-1$
        return true;
    }
}