/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import com.patrikdufresne.managers.ManagerException;
import com.planimod.core.Employee;
import com.planimod.core.Task;
import com.planimod.core.Team;

/**
 * Exception raised when an handle error occurred within the generate planif
 * process.
 * 
 * @author Patrik Dufresne
 */
public class GeneratePlanifException extends ManagerException {

    /**
     * Reason code to identify an operation being cancel by user.
     */
    public static final int CANCEL_BY_USER = 2;

    /**
     * Reason code to identify an employee being locked on a task for which he
     * is not qualied or available. The employee must be unlocked to generate a
     * planif.
     * <p>
     * Should be raised with an employee and a task.
     */
    public static final int EMPLOYEE_LOCKED_UNAVAIL_OR_UNQUALIFY = 1;

    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    /**
     * Reason code raised when the problem is not feasible from the start. This
     * is most probably due to an employee locked on conflicting task or team.
     */
    public static final int STARTING_UNFEASIBLE = 5;

    /**
     * The problem is unfeasible.
     */
    public static final int UNFEASIBLE = 4;

    private Employee employee;

    private int reason;

    private Task task;

    private Team team;

    public GeneratePlanifException(String message, int reason) {
        this(message, reason, null, null, null);
    }

    /**
     * Create a new exception with the reason specified.
     * 
     * @param message the detail message.
     * @param reason the reason.
     * @param employee the related employee or null
     * @param task the related task or null
     */
    public GeneratePlanifException(String message, int reason, Employee employee, Task task) {
        this(message, reason, employee, task, null);
    }

    public GeneratePlanifException(String message, int reason, Employee employee, Task task, Team team) {
        super(message);
        this.reason = reason;
        this.employee = employee;
        this.task = task;
        this.team = team;
    }

    public GeneratePlanifException(String message, int reason, Employee employee, Team team) {
        this(message, reason, employee, null, team);
    }

    /**
     * Return the related employee or null if not set
     * 
     * @return
     */
    public Employee getEmployee() {
        return this.employee;
    }

    /**
     * Return the exception reason.
     * 
     * @return
     */
    public int getReason() {
        return this.reason;
    }

    /**
     * Return the related task or null is not set
     * 
     * @return
     */
    public Task getTask() {
        return this.task;
    }

    public Team getTeam() {
        return this.team;
    }
}
