/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.patrikdufresne.ilp.Linear;
import com.patrikdufresne.ilp.LinearProblem;

/**
 * This runnable is used to find the maximum number of task that may be assigned.
 * <p>
 * This implementation optimize the total number of tasks being assigned whatever the task is. This solver is always
 * feasible.
 * 
 * <pre>
 * max(SUM from{i=0} to {n} SUM from{j=0} to {m} SUM from{k=0} to {p} xijk)
 * </pre>
 * 
 * @author Patrik Dufresne
 */
@SuppressWarnings("unqualified-field-access")
public class MaxAllTaskAssigned extends AbstractConstraintRunnable {

    private static final transient Logger LOGGER = LoggerFactory.getLogger(MaxAllTaskAssigned.class);

    /**
     * The constraint's name.
     */
    private static final String CONSTRAINT_NAME = "MaxAllTaskAssigned: SUM from{i=0} to {n} SUM from{j=0} to {m} SUM from{k=0} to {p} xijk >=%d"; //$NON-NLS-1$

    /**
     * The linear being maximized.
     */
    private Linear linear;

    /**
     * Public constructor.
     * 
     * @param problem
     *            reference to ProblemBuilder
     */
    public MaxAllTaskAssigned(ProblemBuilder problem) {
        super(problem);
        setFeasibilityPumpHeuristic(true);
    }

    /**
     * Create a constraint to maintain the number of assignment.
     */
    @Override
    protected void feasible() {
        // Get objective value
        Number value = Integer.valueOf(pb.lp.getObjectiveValue().intValue());
        LOGGER.info("Maximum number of task assigned: {}", value); //$NON-NLS-1$
        // Sets the number of tasks assigned as a constraint.
        String name = String.format(CONSTRAINT_NAME, value);
        this.constraint = pb.lp.addConstraint(name, this.linear, value, null);
    }

    /**
     * This implementation create the linear to be maximized.
     */
    @Override
    protected boolean setUp() {
        LOGGER.debug("Maximize number of task assigned."); //$NON-NLS-1$
        // Create a linear
        this.linear = pb.lp.createLinear();
        for (int i = 0; i < pb.xijk.length; i++) {
            for (int j = 0; j < pb.xijk[i].length; j++) {
                for (int k = 0; k < pb.xijk[i][j].length; k++) {
                    if (pb.xijk[i][j][k] != null) {
                        this.linear.add(pb.lp.createTerm(1, pb.xijk[i][j][k]));
                    }
                }
            }
        }
        // Set the linear as an objective
        pb.lp.setObjectiveLinear(this.linear);
        pb.lp.setObjectiveDirection(LinearProblem.MAXIMIZE);
        return true;
    }

    /**
     * Reset the objective function.
     */
    @Override
    protected void tearDown() {
        // Remove objective
        pb.lp.setObjectiveLinear(null);
    }

    /**
     * Should never be called.
     */
    @Override
    protected void unfeasible() throws GeneratePlanifException {
        // It's a maximization problem, it' must be feasible.
        throwProblemUnFeasible();
    }
}