/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This runnable is used to minimized the number of swapable task. It's believed to be faster to check for feasibility
 * first and then optimize if required.
 * 
 * @author Patrik Dufresne
 */
public class MinSwappableTask extends SolveRunnable {

    private static final transient Logger LOGGER = LoggerFactory.getLogger(MinSwappableTask.class);

    private CheckSwappableTask checkSwappableTask;

    private MinAllSwappableTask minAllSwappableTask;

    /**
     * public constructor.
     * 
     * @param problem
     *            reference to ProblemBuilder
     */
    public MinSwappableTask(ProblemBuilder problem) {
        super(problem);
    }

    /**
     * This implementation is calling the {@link #engage()} function of the underlying runnable.
     */
    @Override
    public void engage() {
        // Only one of the solve-runnable should be instantiated. Engage it.
        if (this.checkSwappableTask != null) {
            this.checkSwappableTask.engage();
        } else if (this.minAllSwappableTask != null) {
            this.minAllSwappableTask.engage();
        } else {
            throw new RuntimeException("release() should be called first."); //$NON-NLS-1$
        }
    }

    /**
     * This implementation is calling the {@link #release()} of the underlying runnable.
     */
    @Override
    public void release() {
        // Only one of the solve-runnable should be instantiated. Release
        // it.
        if (this.checkSwappableTask != null) {
            this.checkSwappableTask.release();
        } else if (this.minAllSwappableTask != null) {
            this.minAllSwappableTask.release();
        } else {
            throw new RuntimeException("run() should be called first."); //$NON-NLS-1$
        }
    }

    /**
     * This implementation will
     */
    @Override
    protected boolean runSolver() throws GeneratePlanifException {
        this.checkSwappableTask = new CheckSwappableTask(pb);
        this.checkSwappableTask.run();
        if (this.checkSwappableTask.isFeasible()) {
            return true;
        }
        this.checkSwappableTask = null;
        this.minAllSwappableTask = new MinAllSwappableTask(pb);
        this.minAllSwappableTask.run();
        // Should always be feasible.
        return this.minAllSwappableTask.isFeasible();
    }
}