/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.planimod.core.Section;
import com.planimod.core.Task;

class SiTable {

    /**
     * Create a new SiTable.
     * 
     * @param tasks
     *            the list of taskst
     * @return instance of SiTable.
     */
    public static SiTable create(List<Task> tasks) {
        Set<Section> set = new HashSet<Section>();
        for (Task t : tasks) {
            set.add(t.getPosition().getSection());
        }
        List<Section> sections = new ArrayList<Section>(set);
        boolean[][] table = new boolean[set.size()][tasks.size()];
        for (int k = 0; k < tasks.size(); k++) {
            Section section = tasks.get(k).getPosition().getSection();
            table[sections.indexOf(section)][k] = true;
        }
        return new SiTable(sections, table);
    }

    /**
     * Known sections.
     */
    List<Section> sections;

    /**
     * Table to represent the association of task to sections.
     */
    boolean[][] table;

    /**
     * Create a new SiTable.
     * 
     * @param sections
     * @param table
     */
    protected SiTable(List<Section> sections, boolean[][] table) {
        this.sections = sections;
        this.table = table;
    }

    /**
     * Return the coefficient list for the section <code>a</code>.
     * 
     * @param a
     *            the section
     * @return the coefficient list
     */
    public List<Double> get(int a) {
        return new CoefficientList(this.table[a]);
    }

    /**
     * Return the coefficient list for the section specified.
     * 
     * @param section
     *            the section
     * @return the coefficent list or null is the section doesn't exists in this table.
     */
    public List<Double> get(Section section) {
        int a = this.sections.indexOf(section);
        if (a < 0) {
            return null;
        }
        return new CoefficientList(this.table[a]);
    }

    /**
     * Return the section index.
     * 
     * @param section
     *            the section
     * @return the index or -1
     */
    public int indexOf(Section section) {
        return this.sections.indexOf(section);
    }

    /**
     * Check if the task <code>k</code> is part of the section <code>a</code>
     * 
     * @param k
     *            the task
     * @param a
     *            the section
     * @return True if the task is part of the section.
     */
    public boolean kInSa(int k, int a) {
        return this.table[a][k];
    }
}