/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.views.admin.product;

import static com.patrikdufresne.util.Localized._;

import java.io.PrintWriter;
import java.io.StringWriter;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.window.IShellProvider;
import org.eclipse.swt.widgets.Display;

import com.patrikdufresne.jface.dialogs.DetailMessageDialog;
import com.patrikdufresne.managers.ManagerException;
import com.planimod.core.PlanimodManagers;
import com.planimod.core.Product;
import com.planimod.ui.theme.Resources;

public class DuplicateProductAction extends Action {

    /**
     * Product property name
     */
    public static final String PRODUCT = "product"; //$NON-NLS-1$
    /**
     * The product to be duplicated.
     */
    private Product product;
    /**
     * Managers to used to add the element.
     */
    private PlanimodManagers managers;
    /**
     * Selection provider.
     */
    private ISelectionProvider selectionProvider;
    /**
     * Shell provider.
     */
    private IShellProvider shellProvider;

    /**
     * Create a new action.
     * 
     * @param managers
     *            the managers to be used
     * @param shellProvider
     *            a shell provider in case the action need to display a message box.
     * @param selectionProvider
     * 
     */
    public DuplicateProductAction(PlanimodManagers managers, IShellProvider shellProvider, ISelectionProvider selectionProvider) {
        if (managers == null || shellProvider == null) {
            throw new NullPointerException();
        }
        this.managers = managers;
        this.shellProvider = shellProvider;
        this.selectionProvider = selectionProvider;
        setText(_(DuplicateProductAction.class, "DuplicateProduct.text")); //$NON-NLS-1$
        setToolTipText(_(DuplicateProductAction.class, "DuplicateProduct.tooltipText")); //$NON-NLS-1$
        setImageDescriptor(Resources.getImageDescriptor(Resources.ICON_DUPLICATE_16));
        setEnabled(canRun());
    }

    /**
     * This implementation always return true.
     * 
     * @see net.ekwos.gymkhana.ui.views.AbstractAddAction#canCreateObject()
     */
    protected boolean canRun() {
        return (product != null);
    }

    /**
     * Returns the product.
     * 
     * @return
     */
    public Product getProduct() {
        return this.product;
    }

    /**
     * This implementation run the copy of productManager.
     */
    @Override
    public void run() {
        if (!canRun()) {
            return;
        }
        Product newProduct;
        try {
            newProduct = managers.getProductManager().copy(product);
        } catch (ManagerException e) {
            StringWriter sw = new StringWriter();
            e.printStackTrace(new PrintWriter(sw));
            DetailMessageDialog.openDetailWarning(this.shellProvider.getShell(), getText(), _(
                    DuplicateProductAction.class,
                    "DuplicateProduct.cantDuplicateObject"), //$NON-NLS-1$
                    _(DuplicateProductAction.class, "DuplicateProduct.errorOccurred"), //$NON-NLS-1$
                    e);
            return;
        }
        // Select object in the viewer
        final Product finalProduct = newProduct;
        Display.getDefault().asyncExec(new Runnable() {
            @Override
            public void run() {
                selectObjects(finalProduct);
            }
        });
    }

    /**
     * Called to select into the selection provider if define.
     * 
     * @param list
     */
    protected void selectObjects(Product product) {
        if (this.selectionProvider instanceof StructuredViewer) {
            ((StructuredViewer) this.selectionProvider).setSelection(new StructuredSelection(product), true);
        } else if (this.selectionProvider != null) {
            this.selectionProvider.setSelection(new StructuredSelection(product));
        }
    }

    /**
     * Sets the product.
     * 
     * @param product
     */
    public void setProduct(Product product) {
        firePropertyChange(PRODUCT, this.product, this.product = product);
        setEnabled(canRun());
    }
}