/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.views.planif;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Collection;
import java.util.Collections;

import org.eclipse.core.databinding.observable.IObservable;
import org.eclipse.core.databinding.observable.value.IObservableValue;

import com.patrikdufresne.managers.ManagerEvent;
import com.patrikdufresne.managers.ManagerException;
import com.patrikdufresne.managers.databinding.ManagedObjectComputedSet;
import com.planimod.core.Employee;
import com.planimod.core.Task;
import com.planimod.core.planif.GeneratePlanifContext;
import com.planimod.ui.PlanimodPolicy;

/**
 * This observable set provide collection of {@link Employee}. This function is
 * wrapping the function
 * {@link GeneratePlanifContext#listEmployeeByTaskProposed(Task)}.
 * 
 * @author Patrik Dufresne
 * 
 */
public class EmployeeByTaskProposedForContextComputedSet extends ManagedObjectComputedSet {
    /**
     * The managers.
     */
    private GeneratePlanifContext context;

    /**
     * Listen to planif context.
     */
    private PropertyChangeListener listener = new PropertyChangeListener() {
        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            getRealm().exec(new Runnable() {
                @Override
                public void run() {
                    makeDirty();
                }
            });
        }
    };
    /**
     * Observable employee.
     */
    private IObservableValue task;

    /**
     * Create a new observable set of employee.
     * 
     * @param context
     *            the generate planif context
     */
    public EmployeeByTaskProposedForContextComputedSet(GeneratePlanifContext context, IObservableValue task) {
        // Avoid listening to managers events
        super(context.getManagers(), Employee.class, null, new IObservable[] { task });
        this.context = context;
        this.task = task;
    }

    @Override
    public synchronized void dispose() {
        super.dispose();
        this.context = null;
    }

    /**
     * This implementation query the database to get the employees qualify for
     * all null planif event.
     */
    @Override
    protected Collection doList() throws ManagerException {
        if (getTask() == null) {
            return Collections.EMPTY_SET;
        }
        try {
            return this.context.getProposedTasks().employees(getTask());
        } catch (Exception e) {
            PlanimodPolicy.showException(e);
            return Collections.EMPTY_SET;
        }
    }

    /**
     * Return the current task value or null if the observable doesn't contain
     * an {@link Task} object.
     * 
     * @return the current Task or null.
     */
    protected Task getTask() {
        if (this.task.getValue() instanceof Task) {
            return (Task) this.task.getValue();
        }
        return null;
    }

    /**
     * This function notify this class about manager update.
     * <p>
     * This implementation can't determine what required to be updated in the
     * cached, so a full update is required. to do so, the observable set is
     * make dirty.
     */
    @Override
    protected void notifyIfChanged(ManagerEvent event) {
        makeDirty();
    }

    @Override
    protected void startListening() {
        super.startListening();
        this.context.addPropertyChangeListener(GeneratePlanifContext.PROPOSED_TASKS, this.listener);
    }

    @Override
    protected void stopListening() {
        super.stopListening();
        this.context.removePropertyChangeListener(GeneratePlanifContext.PROPOSED_TASKS, this.listener);
    }

}