/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.views.planif;

import java.util.Collection;
import java.util.Collections;
import java.util.Date;

import org.eclipse.core.databinding.observable.IObservable;
import org.eclipse.core.databinding.observable.value.IObservableValue;

import com.patrikdufresne.managers.ManagerException;
import com.patrikdufresne.managers.databinding.ManagedObjectComputedSet;
import com.planimod.core.TimeRanges;
import com.planimod.core.NonAvailabilityManager;
import com.planimod.core.Task;
import com.planimod.core.TaskManager;
import com.planimod.core.PlanimodManagers;

/**
 * This computed set is used to wrap the function
 * {@link NonAvailabilityManager#listProductionEvents(Planif, Date, Date)} and
 * make it observable.
 * 
 * @param managers
 *            the managers to use
 * @param planif
 *            the planif observable
 * @param start
 *            the start date observable
 * @param end
 *            the end date observable
 * @return the observable set
 */
public class TaskComputedSet extends ManagedObjectComputedSet {

    private final IObservableValue end;
    private final IObservableValue start;

    /**
     * Create an observable set.
     * 
     * @param managers
     *            the managers to use
     * @param planif
     *            the planif observable
     * @param start
     *            the start date observable
     * @param end
     *            the end date observable
     * @return the observable set
     */
    public TaskComputedSet(PlanimodManagers managers, IObservableValue start, IObservableValue end) {
        super(managers, Task.class, new IObservable[] { start, end });
        this.start = start;
        this.end = end;
    }

    /**
     * This implementation run the
     * {@link TaskManager#list(Planif, Date, Date)}
     * function.
     */
    @Override
    protected Collection doList() throws ManagerException {
        // Get the current planif value from the planif state.
        if (getStart() == null || getEnd() == null) {
            return Collections.EMPTY_LIST;
        }
        return ((PlanimodManagers) getManagers()).getTaskManager().list(getStart(), getEnd());
    }

    /**
     * Selector to filter according to the list.
     */
    @Override
    protected boolean doSelect(Object element) {
        // Get the current planif value from the planif state.
        Date start = getStart();
        Date end = getEnd();
        if (start == null || end == null) {
            return false;
        }
        return element instanceof Task && TimeRanges.intersectEquals(((Task) element), start, end);
    }

    /**
     * Returns the start date value.
     * 
     * @return the start date or null.
     */
    protected Date getEnd() {
        if (this.end.getValue() instanceof Date) {
            return (Date) this.end.getValue();
        }
        return null;
    }

    /**
     * Returns the end date value.
     * 
     * @return the end date or null.
     */
    protected Date getStart() {
        if (this.start.getValue() instanceof Date) {
            return (Date) this.start.getValue();
        }
        return null;
    }
}